<?php
/**
 * Created by PhpStorm.
 * User: robin
 * Date: 9/17/16
 * Time: 6:02 PM
 */

namespace Luticate\Auth\Controller;


use Luticate\Auth\Dbo\LuBuiltInPermissions;
use Luticate\Utils\Business\LuStringUtils;
use Luticate\Utils\Controller\LuRoute;

class LuAuthController
{
    private $_route = null;

    private $_prefix = null;

    private $_namespace = 'Luticate\Auth\Controller\\';

    /**
     * @param LuRoute $route
     * @param string $prefix
     */
    function __construct(LuRoute $route, $prefix = '/luticate/')
    {
        $this->_route = $route;
        if (!LuStringUtils::endsWith($prefix, '/')) {
            $prefix .= '/';
        }
        $this->_prefix = $prefix;
    }

    function setup()
    {
        $int = LuRoute::REG_UINT;
        $name = "[A-Z0-9_]+";

        $user_id = "{user_id:$int}";
        $group_id = "{group_id:$int}";
        $permission_name = "{permission_name:$name}";
        $setting_name = "{setting_name:$name}";

        $this->post("users/login", "LuUsersController", "login");
        $this->post("users/logout", "LuUsersController", "logout");
        $this->get("users/me", "LuUsersController", "getMe");
        $this->get("users/$user_id", "LuUsersController", "getById", LuBuiltInPermissions::USER_GET);
        $this->get("users", "LuUsersController", "getAll", LuBuiltInPermissions::USER_LIST);
        $this->post("users/add", "LuUsersController", "add", LuBuiltInPermissions::USER_ADD);
        $this->post("users/$user_id/del", "LuUsersController", "del", LuBuiltInPermissions::USER_DEL);
        $this->post("users/$user_id/edit", "LuUsersController", "edit", LuBuiltInPermissions::USER_EDIT);
        $this->post("users/me/edit", "LuUsersController", "editMe", LuBuiltInPermissions::USER_EDIT_ME);
        $this->post("users/$user_id/setPassword", "LuUsersController", "setPassword", LuBuiltInPermissions::USER_SET_PASSWORD);
        $this->post("users/me/setPassword", "LuUsersController", "setPasswordMe", LuBuiltInPermissions::USER_SET_PASSWORD_ME);


        $this->get("groups/$group_id", "LuGroupsController", "getById", LuBuiltInPermissions::GROUP_GET);
        $this->get("groups", "LuGroupsController", "getAll", LuBuiltInPermissions::GROUP_LIST);
        $this->post("groups/add", "LuGroupsController", "add", LuBuiltInPermissions::GROUP_ADD);
        $this->post("groups/$group_id/del", "LuGroupsController", "del", LuBuiltInPermissions::GROUP_DEL);
        $this->post("groups/$group_id/edit", "LuGroupsController", "edit", LuBuiltInPermissions::GROUP_EDIT);

        $this->get("groups/$group_id/users/", "LuGroupsController", "getUsers", LuBuiltInPermissions::GROUP_GET);
        $this->post("groups/$group_id/users/$user_id/add", "LuGroupsController", "addUser", LuBuiltInPermissions::GROUP_USER_ADD);
        $this->post("groups/$group_id/users/$user_id/del", "LuGroupsController", "delUser", LuBuiltInPermissions::GROUP_USER_DEL);


        $this->get("permissions/effective/$user_id/$permission_name", "LuPermissionsController",
            "getEffectivePermission", LuBuiltInPermissions::PERM_EFFECTIVE_GET);
        $this->get("permissions/effective/$user_id", "LuPermissionsController",
            "getAllEffectivePermission", LuBuiltInPermissions::PERM_EFFECTIVE_GET);
        $this->get("permissions/effective/me/$permission_name", "LuPermissionsController", "getEffectivePermissionMe");
        $this->get("permissions/effective/me", "LuPermissionsController", "getAllEffectivePermissionMe");

        $this->get("permissions/$permission_name", "LuPermissionsController", "get", LuBuiltInPermissions::PERM_GET);
        $this->get("permissions", "LuPermissionsController", "getAll", LuBuiltInPermissions::PERM_LIST);
        $this->post("permissions/add/$permission_name", "LuPermissionsController", "add", LuBuiltInPermissions::PERM_ADD);
        $this->post("permissions/$permission_name/del", "LuPermissionsController", "del", LuBuiltInPermissions::PERM_DEL);
        $this->post("permissions/$permission_name/edit", "LuPermissionsController", "edit", LuBuiltInPermissions::PERM_EDIT);

        $this->get("permissions/group/$group_id/$permission_name", "LuPermissionsGroupsController",
            "get", LuBuiltInPermissions::PERM_GROUP_GET);
        $this->get("permissions/group/$group_id", "LuPermissionsGroupsController",
            "getAllByGroup", LuBuiltInPermissions::PERM_GROUP_LIST);
        $this->post("permissions/group/$group_id/add/$permission_name", "LuPermissionsGroupsController",
            "add", LuBuiltInPermissions::PERM_GROUP_ADD);
        $this->post("permissions/group/$group_id/$permission_name/del", "LuPermissionsGroupsController",
            "del", LuBuiltInPermissions::PERM_GROUP_DEL);
        $this->post("permissions/group/$group_id/$permission_name/edit", "LuPermissionsGroupsController",
            "edit", LuBuiltInPermissions::PERM_GROUP_EDIT);

        $this->get("permissions/user/$user_id/$permission_name", "LuPermissionsUsersController",
            "get", LuBuiltInPermissions::PERM_USER_GET);
        $this->get("permissions/user/$user_id", "LuPermissionsUsersController",
            "getAllByUser", LuBuiltInPermissions::PERM_USER_LIST);
        $this->post("permissions/user/$user_id/add/$permission_name", "LuPermissionsUsersController",
            "add", LuBuiltInPermissions::PERM_USER_ADD);
        $this->post("permissions/user/$user_id/$permission_name/del", "LuPermissionsUsersController",
            "del", LuBuiltInPermissions::PERM_USER_DEL);
        $this->post("permissions/user/$user_id/$permission_name/edit", "LuPermissionsUsersController",
            "edit", LuBuiltInPermissions::PERM_USER_EDIT);


        $this->get("settings/effective/$user_id/$setting_name", "LuSettingsController",
            "getEffectiveSetting", LuBuiltInPermissions::SETTING_EFFECTIVE_GET);
        $this->get("settings/effective/$user_id", "LuSettingsController",
            "getAllEffectiveSetting", LuBuiltInPermissions::SETTING_GET);
        $this->get("settings/effective/me/$setting_name", "LuSettingsController", "getEffectiveSettingMe");
        $this->get("settings/effective/me", "LuSettingsController", "getAllEffectiveSettingMe");

        $this->get("settings/$setting_name", "LuSettingsController", "get", LuBuiltInPermissions::SETTING_GET);
        $this->get("settings", "LuSettingsController", "getAll", LuBuiltInPermissions::SETTING_GET);
        $this->post("settings/add", "LuSettingsController", "add", LuBuiltInPermissions::SETTING_ADD);
        $this->post("settings/$setting_name/del", "LuSettingsController", "del", LuBuiltInPermissions::SETTING_DEL);
        $this->post("settings/$setting_name/edit", "LuSettingsController", "edit", LuBuiltInPermissions::SETTING_EDIT);

        $this->get("settings/user/$user_id/$setting_name", "LuSettingsUsersController",
            "get", LuBuiltInPermissions::SETTING_USER_GET);
        $this->get("settings/user/$user_id", "LuSettingsUsersController",
            "getAllByUser", LuBuiltInPermissions::SETTING_USER_GET);
        $this->post("settings/user/$user_id/$setting_name/edit", "LuSettingsUsersController",
            "edit", LuBuiltInPermissions::SETTING_USER_EDIT);
        $this->post("settings/user/me/$setting_name/edit", "LuSettingsUsersController",
            "editMe", LuBuiltInPermissions::SETTING_EDIT_ME);
    }

    function get(string $url, string $controller, string $method, string ...$permissions)
    {
        $this->_route->get($this->_prefix . $url, $this->_namespace . $controller, $method, ["permissions" => $permissions]);
    }

    function post(string $url, string $controller, string $method, string ...$permissions)
    {
        $this->_route->post($this->_prefix . $url, $this->_namespace . $controller, $method, ["permissions" => $permissions]);
    }

    function delete(string $url, string $controller, string $method, string ...$permissions)
    {
        $this->_route->delete($this->_prefix . $url, $this->_namespace . $controller, $method, ["permissions" => $permissions]);
    }
}